@@ -100,7 +100,7 @@
 #' Fleiss1993cont$region <- c("Europe", "Europe", "Asia", "Asia", "Europe")
 #' 
 #' m1 <- metacont(n.psyc, mean.psyc, sd.psyc, n.cont, mean.cont, sd.cont,
-#'   data = Fleiss1993cont, sm = "MD")
+#'   data = Fleiss1993cont, sm = "SMD")
 #' 
 #' mr1 <- metareg(m1, region)
 #' mr1
@@ -178,7 +178,7 @@ bubble.metareg <- function(x,
   sm <- m1$sm
   
   
-  if (backtransf & is.relative.effect(sm)) {
+  if (backtransf & is_relative_effect(sm)) {
     log <- "y"
     TE <- exp(TE)
   }
@@ -186,9 +186,9 @@ bubble.metareg <- function(x,
     log <- ""
   ##
   if (missing(ref)) {
-    if (is.prop(sm) | is.rate(sm) | is.mean(sm))
+    if (is_prop(sm) | is_rate(sm) | is_mean(sm))
       ref <- NA
-    else if (is.relative.effect(sm) & backtransf)
+    else if (is_relative_effect(sm) & backtransf)
       ref <- 1
     else
       ref <- 0
@@ -231,6 +231,8 @@ bubble.metareg <- function(x,
   ##
   if (covar.name %in% names(x$.meta$x$data))
     covar <- x$.meta$x$data[[covar.name]]
+  else if (covar.name %in% names(x$.meta$x))
+    covar <- x$.meta$x[[covar.name]]
   else if (".subgroup" %in% names(x$.meta$x$data))
     covar <- x$.meta$x$data[[".subgroup"]]
   else
@@ -285,7 +287,7 @@ bubble.metareg <- function(x,
     ylim <- range(ys)
   ##
   if (missing(ylab))
-    if (is.relative.effect(sm))
+    if (is_relative_effect(sm))
       ylab <- xlab(sm, backtransf)
     else if (sm == "PRAW")
       ylab <- "Proportion"
