@@ -89,7 +89,7 @@ metainf.meta <- function(x, pooled, sortvar, no = 1, ...) {
   ##
   ##
   chkclass(x, "meta")
-  chksuitable(x, "Leave-one-out method", "netpairwise")
+  chksuitable(x, "Leave-one-out method", c("metamerge", "netpairwise"))
   ##
   x <- updateversion(x)
   ##
@@ -268,7 +268,8 @@ metainf.meta <- function(x, pooled, sortvar, no = 1, ...) {
                    ##
                    exclude = exclude[sel],
                    ##
-                   method = x$method, sm = x$sm,
+                   method = x$method,
+                   sm = x$sm,
                    ##
                    incr = incr.i, method.incr = x$method.incr,
                    allstudies = x$allstudies, MH.exact = x$MH.exact,
@@ -294,7 +295,8 @@ metainf.meta <- function(x, pooled, sortvar, no = 1, ...) {
                     ##
                     exclude = exclude[sel],
                     ##
-                    sm = x$sm, pooledvar = x$pooledvar,
+                    sm = x$sm,
+                    pooledvar = replaceNA(x$pooledvar, gs("pooledvar")),
                     ##
                     method.tau = x$method.tau,
                     tau.preset = x$tau.preset, TE.tau = x$TE.tau,
@@ -351,7 +353,8 @@ metainf.meta <- function(x, pooled, sortvar, no = 1, ...) {
                    ##
                    exclude = exclude[sel],
                    ##
-                   method = x$method, sm = x$sm,
+                   method = x$method,
+                   sm = x$sm,
                    incr = incr.i, method.incr = x$method.incr,
                    model.glmm =
                      if (!is.null(x$model.glmm)) x$model.glmm else "UM.FS",
@@ -392,7 +395,8 @@ metainf.meta <- function(x, pooled, sortvar, no = 1, ...) {
                     ##
                     exclude = exclude[sel],
                     ##
-                    method = x$method, sm = x$sm, null.effect = x$null.effect,
+                    method = x$method,
+                    sm = x$sm, null.effect = x$null.effect,
                     ##
                     incr = incr.i, method.incr = x$method.incr,
                     method.ci = x$method.ci,
@@ -414,7 +418,8 @@ metainf.meta <- function(x, pooled, sortvar, no = 1, ...) {
                     ##
                     exclude = exclude[sel],
                     ##
-                    method = x$method, sm = x$sm, null.effect = x$null.effect,
+                    method = x$method,
+                    sm = x$sm, null.effect = x$null.effect,
                     ##
                     incr = incr.i, method.incr = x$method.incr,
                     ##
@@ -571,9 +576,11 @@ metainf.meta <- function(x, pooled, sortvar, no = 1, ...) {
               overall = TRUE,
               overall.hetstat = TRUE,
               prediction = FALSE,
+              method.predict = x$method.predict,
               backtransf = x$backtransf,
               ##
               method = x$method,
+              method.random = x$method.random,
               ##
               w = c(weight.i, NA, w.s),
               TE.common = NA, seTE.common = NA,
