@@ -402,129 +402,129 @@ extract_outcomes <- function(txt, outcome.type, res,
 
 
 oct2txt <- function(txt) {
-  txt <- gsub("\200", "EUR", txt)
-  txt <- gsub("\202", "'", txt)
-  txt <- gsub("\203", "f", txt)
-  txt <- gsub("\204", '"', txt)
-  txt <- gsub("\205", "...", txt)
-  txt <- gsub("\206", "+", txt)
-  txt <- gsub("\207", "+", txt)
-  txt <- gsub("\210", "^", txt)
-  txt <- gsub("\211", "%%", txt)
-  txt <- gsub("\212", "S", txt)
-  txt <- gsub("\213", "'", txt)
-  txt <- gsub("\214", "OE", txt)
-  txt <- gsub("\216", "Z", txt)
-  txt <- gsub("\221", "'", txt)
-  txt <- gsub("\222", "'", txt)
-  txt <- gsub("\223", '"', txt)
-  txt <- gsub("\224", '"', txt)
-  txt <- gsub("\225", "-", txt)
-  txt <- gsub("\226", "-", txt)
-  txt <- gsub("\227", "-", txt)
-  txt <- gsub("\230", "~", txt)
-  txt <- gsub("\231", "TM", txt)
-  txt <- gsub("\232", "S", txt)
-  txt <- gsub("\233", "'", txt)
-  txt <- gsub("\234", "oe", txt)
-  txt <- gsub("\237", "Y", txt)
-  txt <- gsub("\240", " ", txt)
-  txt <- gsub("\241", "!", txt)
-  txt <- gsub("\242", "cent", txt)
-  txt <- gsub("\243", "Pound", txt)
-  txt <- gsub("\244", "C", txt)
-  txt <- gsub("\245", "Yen", txt)
-  txt <- gsub("\246", "|", txt)
-  txt <- gsub("\247", "S", txt)
-  txt <- gsub("\250", "", txt)
-  txt <- gsub("\251", "(C)", txt)
-  txt <- gsub("\252", "", txt)
-  txt <- gsub("\253", "'", txt)
-  txt <- gsub("\254", "-", txt)
-  txt <- gsub("\255", "-", txt)
-  txt <- gsub("\256", "(R)", txt)
-  txt <- gsub("\257", "-", txt)
-  txt <- gsub("\260", "o", txt)
-  txt <- gsub("\261", "+/-", txt)
-  txt <- gsub("\262", "2", txt)
-  txt <- gsub("\263", "3", txt)
-  txt <- gsub("\264", "", txt)
-  txt <- gsub("\265", "mu", txt)
-  txt <- gsub("\266", "", txt)
-  txt <- gsub("\267", ".", txt)
-  txt <- gsub("\270", " ", txt)
-  txt <- gsub("\271", "1", txt)
-  txt <- gsub("\272", "o", txt)
-  txt <- gsub("\273", '"', txt)
-  txt <- gsub("\274", "1/4", txt)
-  txt <- gsub("\275", "1/2", txt)
-  txt <- gsub("\276", "3/4", txt)
-  txt <- gsub("\277", "?", txt)
-  txt <- gsub("\300", "A", txt)
-  txt <- gsub("\301", "A", txt)
-  txt <- gsub("\302", "A", txt)
-  txt <- gsub("\303", "A", txt)
-  txt <- gsub("\304", "A", txt)
-  txt <- gsub("\305", "A", txt)
-  txt <- gsub("\306", "AE", txt)
-  txt <- gsub("\307", "C", txt)
-  txt <- gsub("\310", "E", txt)
-  txt <- gsub("\311", "E", txt)
-  txt <- gsub("\312", "E", txt)
-  txt <- gsub("\313", "E", txt)
-  txt <- gsub("\314", "I", txt)
-  txt <- gsub("\315", "I", txt)
-  txt <- gsub("\316", "I", txt)
-  txt <- gsub("\317", "I", txt)
-  txt <- gsub("\320", "E", txt)
-  txt <- gsub("\321", "N", txt)
-  txt <- gsub("\322", "O", txt)
-  txt <- gsub("\323", "O", txt)
-  txt <- gsub("\324", "O", txt)
-  txt <- gsub("\325", "O", txt)
-  txt <- gsub("\326", "O", txt)
-  txt <- gsub("\327", "x", txt)
-  txt <- gsub("\330", "O", txt)
-  txt <- gsub("\331", "U", txt)
-  txt <- gsub("\332", "U", txt)
-  txt <- gsub("\333", "U", txt)
-  txt <- gsub("\334", "U", txt)
-  txt <- gsub("\335", "Y", txt)
-  txt <- gsub("\336", "T", txt)
-  txt <- gsub("\337", "ss", txt)
-  txt <- gsub("\340", "a", txt)
-  txt <- gsub("\341", "a", txt)
-  txt <- gsub("\342", "a", txt)
-  txt <- gsub("\343", "a", txt)
-  txt <- gsub("\344", "a", txt)
-  txt <- gsub("\345", "a", txt)
-  txt <- gsub("\346", "a", txt)
-  txt <- gsub("\347", "c", txt)
-  txt <- gsub("\350", "e", txt)
-  txt <- gsub("\351", "e", txt)
-  txt <- gsub("\352", "e", txt)
-  txt <- gsub("\353", "e", txt)
-  txt <- gsub("\354", "i", txt)
-  txt <- gsub("\355", "i", txt)
-  txt <- gsub("\356", "i", txt)
-  txt <- gsub("\357", "i", txt)
-  txt <- gsub("\360", "e", txt)
-  txt <- gsub("\361", "n", txt)
-  txt <- gsub("\362", "o", txt)
-  txt <- gsub("\363", "o", txt)
-  txt <- gsub("\364", "o", txt)
-  txt <- gsub("\365", "o", txt)
-  txt <- gsub("\366", "o", txt)
-  txt <- gsub("\367", "/", txt)
-  txt <- gsub("\370", "o", txt)
-  txt <- gsub("\371", "u", txt)
-  txt <- gsub("\372", "u", txt)
-  txt <- gsub("\373", "u", txt)
-  txt <- gsub("\374", "u", txt)
-  txt <- gsub("\375", "y", txt)
-  txt <- gsub("\376", "t", txt)
-  txt <- gsub("\377", "y", txt)
-  txt <- gsub("\303\237", "AY", txt)
+  txt <- gsub("\200", "EUR", txt, useBytes = TRUE)
+  txt <- gsub("\202", "'", txt, useBytes = TRUE)
+  txt <- gsub("\203", "f", txt, useBytes = TRUE)
+  txt <- gsub("\204", '"', txt, useBytes = TRUE)
+  txt <- gsub("\205", "...", txt, useBytes = TRUE)
+  txt <- gsub("\206", "+", txt, useBytes = TRUE)
+  txt <- gsub("\207", "+", txt, useBytes = TRUE)
+  txt <- gsub("\210", "^", txt, useBytes = TRUE)
+  txt <- gsub("\211", "%%", txt, useBytes = TRUE)
+  txt <- gsub("\212", "S", txt, useBytes = TRUE)
+  txt <- gsub("\213", "'", txt, useBytes = TRUE)
+  txt <- gsub("\214", "OE", txt, useBytes = TRUE)
+  txt <- gsub("\216", "Z", txt, useBytes = TRUE)
+  txt <- gsub("\221", "'", txt, useBytes = TRUE)
+  txt <- gsub("\222", "'", txt, useBytes = TRUE)
+  txt <- gsub("\223", '"', txt, useBytes = TRUE)
+  txt <- gsub("\224", '"', txt, useBytes = TRUE)
+  txt <- gsub("\225", "-", txt, useBytes = TRUE)
+  txt <- gsub("\226", "-", txt, useBytes = TRUE)
+  txt <- gsub("\227", "-", txt, useBytes = TRUE)
+  txt <- gsub("\230", "~", txt, useBytes = TRUE)
+  txt <- gsub("\231", "TM", txt, useBytes = TRUE)
+  txt <- gsub("\232", "S", txt, useBytes = TRUE)
+  txt <- gsub("\233", "'", txt, useBytes = TRUE)
+  txt <- gsub("\234", "oe", txt, useBytes = TRUE)
+  txt <- gsub("\237", "Y", txt, useBytes = TRUE)
+  txt <- gsub("\240", " ", txt, useBytes = TRUE)
+  txt <- gsub("\241", "!", txt, useBytes = TRUE)
+  txt <- gsub("\242", "cent", txt, useBytes = TRUE)
+  txt <- gsub("\243", "Pound", txt, useBytes = TRUE)
+  txt <- gsub("\244", "C", txt, useBytes = TRUE)
+  txt <- gsub("\245", "Yen", txt, useBytes = TRUE)
+  txt <- gsub("\246", "|", txt, useBytes = TRUE)
+  txt <- gsub("\247", "S", txt, useBytes = TRUE)
+  txt <- gsub("\250", "", txt, useBytes = TRUE)
+  txt <- gsub("\251", "(C)", txt, useBytes = TRUE)
+  txt <- gsub("\252", "", txt, useBytes = TRUE)
+  txt <- gsub("\253", "'", txt, useBytes = TRUE)
+  txt <- gsub("\254", "-", txt, useBytes = TRUE)
+  txt <- gsub("\255", "-", txt, useBytes = TRUE)
+  txt <- gsub("\256", "(R)", txt, useBytes = TRUE)
+  txt <- gsub("\257", "-", txt, useBytes = TRUE)
+  txt <- gsub("\260", "o", txt, useBytes = TRUE)
+  txt <- gsub("\261", "+/-", txt, useBytes = TRUE)
+  txt <- gsub("\262", "2", txt, useBytes = TRUE)
+  txt <- gsub("\263", "3", txt, useBytes = TRUE)
+  txt <- gsub("\264", "", txt, useBytes = TRUE)
+  txt <- gsub("\265", "mu", txt, useBytes = TRUE)
+  txt <- gsub("\266", "", txt, useBytes = TRUE)
+  txt <- gsub("\267", ".", txt, useBytes = TRUE)
+  txt <- gsub("\270", " ", txt, useBytes = TRUE)
+  txt <- gsub("\271", "1", txt, useBytes = TRUE)
+  txt <- gsub("\272", "o", txt, useBytes = TRUE)
+  txt <- gsub("\273", '"', txt, useBytes = TRUE)
+  txt <- gsub("\274", "1/4", txt, useBytes = TRUE)
+  txt <- gsub("\275", "1/2", txt, useBytes = TRUE)
+  txt <- gsub("\276", "3/4", txt, useBytes = TRUE)
+  txt <- gsub("\277", "?", txt, useBytes = TRUE)
+  txt <- gsub("\300", "A", txt, useBytes = TRUE)
+  txt <- gsub("\301", "A", txt, useBytes = TRUE)
+  txt <- gsub("\302", "A", txt, useBytes = TRUE)
+  txt <- gsub("\303", "A", txt, useBytes = TRUE)
+  txt <- gsub("\304", "A", txt, useBytes = TRUE)
+  txt <- gsub("\305", "A", txt, useBytes = TRUE)
+  txt <- gsub("\306", "AE", txt, useBytes = TRUE)
+  txt <- gsub("\307", "C", txt, useBytes = TRUE)
+  txt <- gsub("\310", "E", txt, useBytes = TRUE)
+  txt <- gsub("\311", "E", txt, useBytes = TRUE)
+  txt <- gsub("\312", "E", txt, useBytes = TRUE)
+  txt <- gsub("\313", "E", txt, useBytes = TRUE)
+  txt <- gsub("\314", "I", txt, useBytes = TRUE)
+  txt <- gsub("\315", "I", txt, useBytes = TRUE)
+  txt <- gsub("\316", "I", txt, useBytes = TRUE)
+  txt <- gsub("\317", "I", txt, useBytes = TRUE)
+  txt <- gsub("\320", "E", txt, useBytes = TRUE)
+  txt <- gsub("\321", "N", txt, useBytes = TRUE)
+  txt <- gsub("\322", "O", txt, useBytes = TRUE)
+  txt <- gsub("\323", "O", txt, useBytes = TRUE)
+  txt <- gsub("\324", "O", txt, useBytes = TRUE)
+  txt <- gsub("\325", "O", txt, useBytes = TRUE)
+  txt <- gsub("\326", "O", txt, useBytes = TRUE)
+  txt <- gsub("\327", "x", txt, useBytes = TRUE)
+  txt <- gsub("\330", "O", txt, useBytes = TRUE)
+  txt <- gsub("\331", "U", txt, useBytes = TRUE)
+  txt <- gsub("\332", "U", txt, useBytes = TRUE)
+  txt <- gsub("\333", "U", txt, useBytes = TRUE)
+  txt <- gsub("\334", "U", txt, useBytes = TRUE)
+  txt <- gsub("\335", "Y", txt, useBytes = TRUE)
+  txt <- gsub("\336", "T", txt, useBytes = TRUE)
+  txt <- gsub("\337", "ss", txt, useBytes = TRUE)
+  txt <- gsub("\340", "a", txt, useBytes = TRUE)
+  txt <- gsub("\341", "a", txt, useBytes = TRUE)
+  txt <- gsub("\342", "a", txt, useBytes = TRUE)
+  txt <- gsub("\343", "a", txt, useBytes = TRUE)
+  txt <- gsub("\344", "a", txt, useBytes = TRUE)
+  txt <- gsub("\345", "a", txt, useBytes = TRUE)
+  txt <- gsub("\346", "a", txt, useBytes = TRUE)
+  txt <- gsub("\347", "c", txt, useBytes = TRUE)
+  txt <- gsub("\350", "e", txt, useBytes = TRUE)
+  txt <- gsub("\351", "e", txt, useBytes = TRUE)
+  txt <- gsub("\352", "e", txt, useBytes = TRUE)
+  txt <- gsub("\353", "e", txt, useBytes = TRUE)
+  txt <- gsub("\354", "i", txt, useBytes = TRUE)
+  txt <- gsub("\355", "i", txt, useBytes = TRUE)
+  txt <- gsub("\356", "i", txt, useBytes = TRUE)
+  txt <- gsub("\357", "i", txt, useBytes = TRUE)
+  txt <- gsub("\360", "e", txt, useBytes = TRUE)
+  txt <- gsub("\361", "n", txt, useBytes = TRUE)
+  txt <- gsub("\362", "o", txt, useBytes = TRUE)
+  txt <- gsub("\363", "o", txt, useBytes = TRUE)
+  txt <- gsub("\364", "o", txt, useBytes = TRUE)
+  txt <- gsub("\365", "o", txt, useBytes = TRUE)
+  txt <- gsub("\366", "o", txt, useBytes = TRUE)
+  txt <- gsub("\367", "/", txt, useBytes = TRUE)
+  txt <- gsub("\370", "o", txt, useBytes = TRUE)
+  txt <- gsub("\371", "u", txt, useBytes = TRUE)
+  txt <- gsub("\372", "u", txt, useBytes = TRUE)
+  txt <- gsub("\373", "u", txt, useBytes = TRUE)
+  txt <- gsub("\374", "u", txt, useBytes = TRUE)
+  txt <- gsub("\375", "y", txt, useBytes = TRUE)
+  txt <- gsub("\376", "t", txt, useBytes = TRUE)
+  txt <- gsub("\377", "y", txt, useBytes = TRUE)
+  txt <- gsub("\303\237", "AY", txt, useBytes = TRUE)
   ##
   txt
 }
@@ -1166,7 +1166,7 @@ read.rm5.rm5 <- function(file, title, numbers.in.labels = TRUE, debug = 0) {
   res$label.left <- as.character(res$label.left)
   res$label.right <- as.character(res$label.right)
   ##
-  ## sel.rel <- is.relative.effect(res$sm)
+  ## sel.rel <- is_relative_effect(res$sm)
   ## res$TE[sel.rel] <- log(res$TE[sel.rel])
   ## res$lower.TE[sel.rel] <- log(res$lower.TE[sel.rel])
   ## res$upper.TE[sel.rel] <- log(res$upper.TE[sel.rel])
